'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _dec, _class, _class2, _temp, _class3, _temp2;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _formsyReact = require('formsy-react');

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _debounce = require('./debounce');

var _debounce2 = _interopRequireDefault(_debounce);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// wrap async validation promise so the passed error can have information about validator name
function wrapPromise(promise, validatorName) {
  return new Promise(function (resolve, reject) {
    return promise.then(function (payload) {
      resolve(payload);
    }, function (err) {
      reject({ validatorName: validatorName, error: err });
    });
  });
}

var BaseInput = (_dec = (0, _formsyReact.Decorator)(), _dec(_class = (_temp = _class2 = function (_Component) {
  _inherits(BaseInput, _Component);

  function BaseInput(props, context) {
    _classCallCheck(this, BaseInput);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(BaseInput).call(this, props, context));

    _this.state = {
      isValidating: false,
      responseError: null
    };
    _this.debouncedHandleAsyncValidations = (0, _debounce2.default)(function () {
      _this.handleAsyncValidations.call(_this, _this.state.fieldValue);
    }, 300);
    return _this;
  }

  _createClass(BaseInput, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      if (this.props.asyncValidations) {
        this.context.formsyWrapper.setAsyncValidationState(false);
      }
    }
  }, {
    key: 'handleChange',
    value: function handleChange(e) {
      // hide response error message if any
      this.setState({ responseError: null, hasAsyncError: false });

      var value = e.target.value;
      var asyncValidations = this.props.asyncValidations;


      this.props.setValue(value);
      this.setState({ fieldValue: value });

      // only start asynchronous validations when synchronous validations scucess
      if (asyncValidations && this.props.isValid()) {
        this.debouncedHandleAsyncValidations();
      }
    }
  }, {
    key: 'handleAsyncValidations',
    value: function handleAsyncValidations(value) {
      var _this2 = this;

      if (!value) {
        return;
      }
      var asyncValidations = this.props.asyncValidations;
      var formsyWrapper = this.context.formsyWrapper;

      var asyncValidationRules = formsyWrapper.getAsyncValidationRules();
      var asyncValidationPromise = [];

      this.setState({ isValidating: true });

      if (typeof asyncValidations === 'string') {
        asyncValidations.split(',').forEach(function (validatorName) {
          var validationMethod = asyncValidationRules[validatorName];
          if (validationMethod) {
            var p = wrapPromise(validationMethod(value), validatorName);
            asyncValidationPromise.push(p);
          }
        });
      } else if ((typeof asyncValidations === 'undefined' ? 'undefined' : _typeof(asyncValidations)) === 'object') {
        Object.keys(asyncValidations).forEach(function (validatorName) {
          var validationMethod = asyncValidationRules[validatorName];
          if (validationMethod) {
            var p = wrapPromise(validationMethod(value, asyncValidations[validatorName]), validatorName);
            asyncValidationPromise.push(p);
          }
        });
      }
      return Promise.all(asyncValidationPromise).then(function () {
        formsyWrapper.setAsyncValidationState(true);
        return _this2.setState({ isValidating: false, hasAsyncError: false });
      }).catch(function (err) {
        var msg = _this2.getAsyncErrorMessage(err.validatorName);

        formsyWrapper.setAsyncValidationState(false);

        return _this2.setState({ isValidating: false, responseError: msg, hasAsyncError: true });
      });
    }
  }, {
    key: 'getAsyncErrorMessage',
    value: function getAsyncErrorMessage(validatorName) {
      var _props = this.props;
      var asyncValidationErrors = _props.asyncValidationErrors;
      var locale = _props.locale;

      if (asyncValidationErrors && asyncValidationErrors[validatorName]) {
        return locale && asyncValidationErrors[validatorName][locale] ? asyncValidationErrors[validatorName][locale] : asyncValidationErrors[validatorName];
      }
      return null;
    }
  }, {
    key: 'render',
    value: function render() {
      var _props2 = this.props;
      var type = _props2.type;
      var name = _props2.name;
      var className = _props2.className;
      var showRequired = _props2.showRequired;
      var showError = _props2.showError;
      var getValue = _props2.getValue;
      var getErrorMessage = _props2.getErrorMessage;

      var classes = (0, _classnames2.default)(className, {
        'error': showError() || this.state.hasAsyncError,
        'required': showRequired(),
        'validating': this.state.isValidating
      });
      var errorMessage = showError() ? getErrorMessage() : this.state.responseError;

      return _react2.default.createElement(
        'div',
        { className: classes },
        _react2.default.createElement('input', {
          type: type || 'text',
          name: name,
          value: getValue() || '',
          onChange: this.handleChange.bind(this) }),
        _react2.default.createElement(
          'span',
          { className: 'errorInfo' },
          errorMessage
        )
      );
    }
  }]);

  return BaseInput;
}(_react.Component), _class2.propTypes = {
  locale: _react.PropTypes.string,
  value: _react.PropTypes.any,
  type: _react.PropTypes.string,
  name: _react.PropTypes.string,
  autoComplete: _react.PropTypes.string,
  setValue: _react.PropTypes.func,
  getValue: _react.PropTypes.func,
  maxLength: _react.PropTypes.number,
  asyncValidations: _react.PropTypes.oneOfType([_react.PropTypes.string, _react.PropTypes.object]),
  asyncValidationErrors: _react.PropTypes.object,
  className: _react.PropTypes.string,
  showRequired: _react.PropTypes.func,
  showError: _react.PropTypes.func,
  getErrorMessage: _react.PropTypes.func
}, _class2.contextTypes = {
  formsyWrapper: _react.PropTypes.object
}, _temp)) || _class);

/*
*
*/

var FormsyInput = (_temp2 = _class3 = function (_Component2) {
  _inherits(FormsyInput, _Component2);

  function FormsyInput() {
    _classCallCheck(this, FormsyInput);

    return _possibleConstructorReturn(this, Object.getPrototypeOf(FormsyInput).apply(this, arguments));
  }

  _createClass(FormsyInput, [{
    key: 'resetValue',
    value: function resetValue() {
      this.refs.baseInput.resetValue();
    }
  }, {
    key: 'render',
    value: function render() {
      var _props3 = this.props;
      var validationError = _props3.validationError;
      var validationErrors = _props3.validationErrors;
      var locale = _props3.locale;

      var parsedObj, newProps;

      if (locale && validationErrors) {
        parsedObj = {};
        Object.keys(validationErrors).forEach(function (validatorName) {
          parsedObj[validatorName] = validationErrors[validatorName][locale] ? validationErrors[validatorName][locale] : validationErrors[validatorName];
        });
        newProps = _extends({}, this.props, { validationErrors: parsedObj });
        return _react2.default.createElement(BaseInput, _extends({}, newProps, { ref: 'baseInput' }));
      }

      if (locale && validationError) {
        if (typeof validationError === 'string') {
          parsedObj = validationError;
        }
        if ((typeof validationError === 'undefined' ? 'undefined' : _typeof(validationError)) === 'object') {
          parsedObj = validationError[locale] ? validationError[locale] : validationError.toString();
        }
        newProps = _extends({}, this.props, { validationError: parsedObj });
        return _react2.default.createElement(BaseInput, _extends({}, newProps, { ref: 'baseInput' }));
      }

      return _react2.default.createElement(BaseInput, _extends({}, this.props, {
        ref: 'baseInput' }));
    }
  }]);

  return FormsyInput;
}(_react.Component), _class3.propTypes = {
  locale: _react.PropTypes.string,
  validations: _react.PropTypes.oneOfType([_react.PropTypes.string, _react.PropTypes.object]),
  validationError: _react.PropTypes.oneOfType([_react.PropTypes.string, _react.PropTypes.object]),
  validationErrors: _react.PropTypes.object
}, _temp2);
exports.default = FormsyInput;